from ruleau import All, ApiAdapter, OverrideLevel, execute, rule


@rule(rule_id="rul_child", name="Has children")
def has_children(_, payload):
    """
    Checks whether the custom has any children.

    >>> has_children(None, {"data": {"number_of_children": 0}})
    False
    >>> has_children(None, {"data": {"number_of_children": 1}})
    True
    """
    return payload["data"]["number_of_children"] > 0


@rule(rule_id="rul_cap", name="Have sufficient capital", depends_on=[has_children])
def has_sufficient_capital(context_test, payload):
    """
    Checks that the client has sufficient capital, considering the number of
    children they have.

    >>> from ruleau import mock_context
    >>> context = mock_context({
    ...     "has_children": True
    ... })
    >>> has_sufficient_capital(context, {"data": {"capital": 10_000}})
    False

    >>> from ruleau import mock_context
    >>> context = mock_context({
    ...     "has_children": False
    ... })
    >>> has_sufficient_capital(context, {"data": {"capital": 10_000}})
    True
    """
    if context_test.get_result(has_children):
        return payload["data"]["capital"] > 12_000
    else:
        return payload["data"]["capital"] > 8_000


@rule(rule_id="rul_01", name="kyc_threshold", override_level=OverrideLevel.NO_OVERRIDE)
def kyc_risk_greater_than_threshold(_, payload):
    """
    Know Your Customer (KYC) score must be greater than the threshold,
    in this case greater than `LOW`

    :Override Guidance: kyc_threshold to mark this failure into review

    >>> kyc_risk_greater_than_threshold(None, {"data": {"kyc": "HIGH"}})
    False
    >>> kyc_risk_greater_than_threshold(None, {"data": {"kyc": "LOW"}})
    True
    """
    return payload["data"]["kyc"] == "LOW"


@rule(rule_id="rul_02", name="fico_score")
def fico_score_greater_than_threshold(_, payload):
    """
    FICO score must be greater than 630

    :Owner: Penny Farthing
    :Override Guidance: Feel free to override in almost any circumstance

    >>> fico_score_greater_than_threshold(None, {"data": {"fico_score": 400}})
    False
    >>> fico_score_greater_than_threshold(None, {"data": {"fico_score": 630}})
    False
    >>> fico_score_greater_than_threshold(None, {"data": {"fico_score": 650}})
    True
    """
    return payload["data"]["fico_score"] > 630


@rule(rule_id="rul_ccjs_required", name="CCJS check required")
def ccjs_check_required(_, payload):
    return payload["data"]["ccjs_required"]


@rule(rule_id="rul_03", name="no_ccjs", run_if=ccjs_check_required)
def has_no_ccjs(_, payload):
    """
    Make sure customer has no county court judgements
    >>> has_no_ccjs(None, {"data": {"ccjs": []}})
    True
    >>> has_no_ccjs(None, {"data": {"ccjs": ["Example CCJ"]}})
    False
    >>> has_no_ccjs(None, {"data": {"ccjs": [{"example": "CCJ Object"}]}})
    False
    """
    return len(payload["data"]["ccjs"]) == 0



class RuleRequiresNameException(Exception):
    """Exception raised if a rule doesn't have a human readable name"""


class RuleRequiresIdException(Exception):
    """Exception raised if rule doesn't have an id"""


class MethodNotAllowedException(Exception):
    """Exception raised if a forbidden RuleauDict method is called"""


class CaseIdRequiredException(Exception):
    """Exception raised if a json path for case identifier is not found"""


class CannotOverrideException(Exception):
    """Exception raised if a API tries to override a rule marked as NO_OVERRIDE"""


class DuplicateRuleNameException(Exception):
    """Exception raised if more than 1 rule has same name"""


class RuleIdIllegalCharacterException(Exception):
    """Exception raise if Rule name contains illegal characters"""


class RuleFunctionNameAttributeCollisionException(Exception):
    """If the provided rule name collides with a rule attribute"""


class DuplicateRuleIdException(Exception):
    """Exception raised if more than 1 rule has same name"""


class APIException(Exception):
    """Generic exception for API request failure"""


class ConditionalDependencyReusedException(Exception):
    """Exception raised if the rule has duplicate dependencies"""


class RuleErrorException(Exception):
    """Exception raised when a rule raises an exception"""

    def __init__(self, rule, rule_exception):
        self.rule = rule
        self.rule_exception = rule_exception


class RunIfRuleHasRunIfException(Exception):
    """Exception raise if a run_if dependency of a rule also has a run_if."""


class IncorrectKwargsForDoctests(Exception):
    """The required keynames context and payload have not been provided"""

    def __init__(self, rule):
        self.rule_name = rule.name
        self.rule_id = rule.id
        self.message = (
            f"The rule with id: {self.rule_id} - name: {self.rule_name} has not "
            f"been defined with the correct keyword argument names.\n Please "
            f"change the rule argument names to context and payload."
        )
        super().__init__(self.message)
